@@ -35,14 +35,17 @@ GNU General Public License for more details.
 #include "pencildef.h"
 #include "pencilsettings.h"
 #include "object.h"
-#include "filemanager.h"
 #include "editor.h"
+
+#include "filemanager.h"
 #include "colormanager.h"
 #include "layermanager.h"
-#include "layercamera.h"
 #include "toolmanager.h"
 #include "playbackmanager.h"
 #include "soundmanager.h"
+#include "viewmanager.h"
+
+#include "layercamera.h"
 #include "actioncommands.h"
 #include "fileformat.h"     //contains constants used by Pencil File Format
 #include "util.h"
@@ -51,12 +54,14 @@ GNU General Public License for more details.
 // app headers
 #include "scribblearea.h"
 #include "colorbox.h"
+#include "colorinspector.h"
 #include "colorpalettewidget.h"
 #include "displayoptionwidget.h"
 #include "tooloptionwidget.h"
 #include "preferencesdialog.h"
 #include "timeline.h"
 #include "toolbox.h"
+
 //#include "preview.h"
 #include "timeline2.h"
 #include "errordialog.h"
@@ -137,9 +142,13 @@ void MainWindow2::createDockWidgets()
     mTimeLine = new TimeLine(this);
     mTimeLine->setObjectName("TimeLine");
 
-    mColorWheel = new ColorBox(this);
-    mColorWheel->setToolTip(tr("color palette:<br>use <b>(C)</b><br>toggle at cursor"));
-    mColorWheel->setObjectName("ColorWheel");
+    mColorBox = new ColorBox(this);
+    mColorBox->setToolTip(tr("color palette:<br>use <b>(C)</b><br>toggle at cursor"));
+    mColorBox->setObjectName("ColorWheel");
+
+    mColorInspector = new ColorInspector(this);
+    mColorInspector->setToolTip(tr("Color inspector"));
+    mColorInspector->setObjectName("Color Inspector");
 
     mColorPalette = new ColorPaletteWidget(this);
     mColorPalette->setObjectName("ColorPalette");
@@ -161,12 +170,14 @@ void MainWindow2::createDockWidgets()
 
     mDockWidgets
         << mTimeLine
-        << mColorWheel
+        << mColorBox
+        << mColorInspector
         << mColorPalette
         << mDisplayOptionWidget
         << mToolOptions
         << mToolBox;
 
+//    mColorInspector->setFloating(true);
     mStartIcon = QIcon(":icons/controls/play.png");
     mStopIcon = QIcon(":icons/controls/stop.png");
 
@@ -182,11 +193,12 @@ void MainWindow2::createDockWidgets()
         qDebug() << "Init Dock widget: " << pWidget->objectName();
     }
 
-    addDockWidget(Qt::RightDockWidgetArea, mColorWheel);
+    addDockWidget(Qt::RightDockWidgetArea, mColorBox);
+    addDockWidget(Qt::RightDockWidgetArea, mColorInspector);
     addDockWidget(Qt::RightDockWidgetArea, mColorPalette);
-    addDockWidget(Qt::RightDockWidgetArea, mDisplayOptionWidget);
     addDockWidget(Qt::LeftDockWidgetArea, mToolBox);
     addDockWidget(Qt::LeftDockWidgetArea, mToolOptions);
+    addDockWidget(Qt::LeftDockWidgetArea, mDisplayOptionWidget);
     addDockWidget(Qt::BottomDockWidgetArea, mTimeLine);
     setDockNestingEnabled(true);
     //addDockWidget( Qt::BottomDockWidgetArea, mTimeline2);
@@ -201,7 +213,8 @@ void MainWindow2::createDockWidgets()
 
     makeConnections(mEditor);
     makeConnections(mEditor, mTimeLine);
-    makeConnections(mEditor, mColorWheel);
+    makeConnections(mEditor, mColorBox);
+    makeConnections(mEditor, mColorInspector);
     makeConnections(mEditor, mColorPalette);
     makeConnections(mEditor, mToolOptions);
 
@@ -226,6 +239,7 @@ void MainWindow2::createMenus()
     connect(ui->actionExport_Image, &QAction::triggered, mCommands, &ActionCommands::exportImage);
     connect(ui->actionExport_ImageSeq, &QAction::triggered, mCommands, &ActionCommands::exportImageSequence);
     connect(ui->actionExport_Movie, &QAction::triggered, mCommands, &ActionCommands::exportMovie);
+    connect(ui->actionExport_Animated_GIF, &QAction::triggered, mCommands, &ActionCommands::exportGif);
 
     connect(ui->actionExport_Palette, &QAction::triggered, this, &MainWindow2::exportPalette);
 
@@ -233,6 +247,7 @@ void MainWindow2::createMenus()
     //connect( ui->actionExport_Svg_Image, &QAction::triggered, editor, &Editor::saveSvg );
     connect(ui->actionImport_Image, &QAction::triggered, this, &MainWindow2::importImage);
     connect(ui->actionImport_ImageSeq, &QAction::triggered, this, &MainWindow2::importImageSequence);
+    connect(ui->actionImport_Gif, &QAction::triggered, this, &MainWindow2::importGIF);
     connect(ui->actionImport_Movie, &QAction::triggered, this, &MainWindow2::importMovie);
 
     connect(ui->actionImport_Sound, &QAction::triggered, mCommands, &ActionCommands::importSound);
@@ -288,6 +303,7 @@ void MainWindow2::createMenus()
     connect(pPlaybackManager, &PlaybackManager::rangedPlaybackStateChanged, mTimeLine, &TimeLine::setRangeState);
     connect(pPlaybackManager, &PlaybackManager::playStateChanged, mTimeLine, &TimeLine::setPlaying);
     connect(pPlaybackManager, &PlaybackManager::playStateChanged, this, &MainWindow2::changePlayState);
+    connect(pPlaybackManager, &PlaybackManager::playStateChanged, mEditor, &Editor::updateCurrentFrame);
 
     connect(ui->actionAdd_Frame, &QAction::triggered, mCommands, &ActionCommands::addNewKey);
     connect(ui->actionRemove_Frame, &QAction::triggered, mCommands, &ActionCommands::removeKey);
@@ -320,10 +336,11 @@ void MainWindow2::createMenus()
     {
         mToolBox->toggleViewAction(),
         mToolOptions->toggleViewAction(),
-        mColorWheel->toggleViewAction(),
+        mColorBox->toggleViewAction(),
         mColorPalette->toggleViewAction(),
         mTimeLine->toggleViewAction(),
-        mDisplayOptionWidget->toggleViewAction()
+        mDisplayOptionWidget->toggleViewAction(),
+        mColorInspector->toggleViewAction()
     };
 
     for (QAction* action : actions)
@@ -416,9 +433,9 @@ void MainWindow2::tabletEvent(QTabletEvent* event)
     event->ignore();
 }
 
-void MainWindow2::newDocument()
+void MainWindow2::newDocument(bool force)
 {
-    if (maybeSave())
+    if (force || maybeSave())
     {
         Object* object = new Object();
         object->init();
@@ -452,14 +469,8 @@ void MainWindow2::openDocument()
             return;
         }
 
-        bool ok = openObject(fileName);
-        if (!ok)
-        {
-            QMessageBox::warning(this, tr("Warning"), tr("Pencil cannot read this file. If you want to import images, use the command import."));
-            newDocument();
-        }
+        openObject(fileName, false);
     }
-    updateSaveState();
 }
 
 bool MainWindow2::saveAsNewDocument()
@@ -480,16 +491,53 @@ bool MainWindow2::saveAsNewDocument()
 
 void MainWindow2::openFile(QString filename)
 {
-    bool ok = openObject(filename);
-    if (!ok)
-    {
-        QMessageBox::warning(this, tr("Warning"), tr("Pencil cannot read this file. If you want to import images, use the command import."));
-        newDocument();
-    }
+    openObject(filename, true);
 }
 
-bool MainWindow2::openObject(QString strFilePath)
+bool MainWindow2::openObject(QString strFilePath, bool checkForChanges)
 {
+    if(checkForChanges && !maybeSave()) return false; // Open cancelled by user
+
+    // Check for potential issues with the file
+    QFileInfo fileInfo(strFilePath);
+    if (fileInfo.isDir())
+    {
+        ErrorDialog errorDialog(tr("Could not open file"),
+                                tr("The file you have selected is a directory, so we are unable to open it. "
+                                   "If you are are trying to open a project that uses the old structure, please "
+                                   "open the file ending with .pcl, not the data folder."),
+                                QString("Raw file path: %1\nResolved file path: %2").arg(strFilePath, fileInfo.absoluteFilePath()));
+        errorDialog.exec();
+        return false;
+    }
+    if (!fileInfo.exists())
+    {
+        ErrorDialog errorDialog(tr("Could not open file"),
+                                tr("The file you have selected does not exist, so we are unable to open it. Please check "
+                                   "to make sure that you've entered the correct path and that the file is accessible and try again."),
+                                QString("Raw file path: %1\nResolved file path: %2").arg(strFilePath, fileInfo.absoluteFilePath()));
+        errorDialog.exec();
+        return false;
+    }
+    if (!fileInfo.isReadable())
+    {
+        ErrorDialog errorDialog(tr("Could not open file"),
+                                tr("This program does not have permission to read the file you have selected. "
+                                   "Please check that you have read permissions for this file and try again."),
+                                QString("Raw file path: %1\nResolved file path: %2\nPermissions: 0x%3") \
+                                .arg(strFilePath, fileInfo.absoluteFilePath(), QString::number(fileInfo.permissions(), 16)));
+        errorDialog.exec();
+        return false;
+    }
+    if (!fileInfo.isWritable())
+    {
+        QMessageBox::warning(this, tr("Warning"),
+                             tr("This program does not currently have permission to write to the file you have selected. "
+                                "Please make sure you have write permission for this file before attempting to save it. "
+                                "Alternatively, you can use the Save As... menu option to save to a writable location."),
+                             QMessageBox::Ok);
+    }
+
     QProgressDialog progress(tr("Opening document..."), tr("Abort"), 0, 100, this);
 
     // Don't show progress bar if running without a GUI (aka. when rendering from command line)
@@ -517,8 +565,27 @@ bool MainWindow2::openObject(QString strFilePath)
 
     Object* object = fm.load(strFilePath);
 
-    if (object == nullptr || !fm.error().ok())
+    if (!fm.error().ok())
     {
+        Status error = fm.error();
+        DebugDetails dd;
+        dd << QString("Raw file path: ").append(strFilePath)
+           << QString("Resolved file path: ").append(fileInfo.absoluteFilePath());
+        dd.collect(error.details());
+        ErrorDialog errorDialog(error.title(),
+                                error.description(),
+                                dd.str());
+        errorDialog.exec();
+        newDocument(true);
+        return false;
+    }
+
+    if (object == nullptr) {
+        ErrorDialog errorDialog(tr("Could not open file"),
+                                tr("An unknown error occurred while trying to load the file and we are not able to load your file."),
+                                QString("Raw file path: %1\nResolved file path: %2").arg(strFilePath, fileInfo.absoluteFilePath()));
+        errorDialog.exec();
+        newDocument(true);
         return false;
     }
 
@@ -538,11 +605,12 @@ bool MainWindow2::openObject(QString strFilePath)
 
     // Refresh the Palette
     mColorPalette->refreshColorList();
-    mEditor->color()->setColorNumber(0);
     mEditor->layers()->notifyAnimationLengthChanged();
 
     progress.setValue(progress.maximum());
 
+    updateSaveState();
+
     return true;
 }
 
@@ -584,7 +652,7 @@ bool MainWindow2::saveObject(QString strSavedFileName)
         if (eLog.open(QIODevice::WriteOnly | QIODevice::Text))
         {
             QTextStream out(&eLog);
-            out << st.details().replace("<br>", "\n", Qt::CaseInsensitive);
+            out << st.details().str(); // .replace("<br>", "\n", Qt::CaseInsensitive);
         }
         eLog.close();
 
@@ -592,7 +660,7 @@ bool MainWindow2::saveObject(QString strSavedFileName)
                                 st.description().append(tr("<br><br>An error has occurred and your file may not have saved successfully."
                                                            "If you believe that this error is an issue with Pencil2D, please create a new issue at:"
                                                            "<br><a href='https://github.com/pencil2d/pencil/issues'>https://github.com/pencil2d/pencil/issues</a><br>"
-                                                           "Please be sure to include the following details in your issue:")), st.details());
+                                                           "Please be sure to include the following details in your issue:")), st.details().html());
         errorDialog.exec();
         return false;
     }
@@ -721,21 +789,18 @@ void MainWindow2::importImageSequence()
     progress.setMaximum(totalImagesToImport);
     int imagesImportedSoFar = 0;
 
+    QString failedFiles;
+    bool failedImport = false;
     for (QString strImgFile : files)
     {
         QString strImgFileLower = strImgFile.toLower();
+
         if (strImgFileLower.endsWith(".png") ||
             strImgFileLower.endsWith(".jpg") ||
             strImgFileLower.endsWith(".jpeg") ||
-            strImgFileLower.endsWith(".tif") ||
-            strImgFileLower.endsWith(".tiff") ||
             strImgFileLower.endsWith(".bmp"))
         {
             mEditor->importImage(strImgFile);
-            for (int i = 1; i < number; i++)
-            {
-                mEditor->scrubForward();
-            }
 
             imagesImportedSoFar++;
             progress.setValue(imagesImportedSoFar);
@@ -745,8 +810,29 @@ void MainWindow2::importImageSequence()
             {
                 break;
             }
+        } else {
+            failedFiles += strImgFile + "\n";
+            if (!failedImport)
+            {
+                failedImport = true;
+            }
+        }
+
+        for (int i = 1; i < number; i++)
+        {
+            mEditor->scrubForward();
         }
     }
+
+    if (failedImport)
+    {
+        QMessageBox::warning(this,
+                             tr("Warning"),
+                             tr("was unable to import") + failedFiles,
+                             QMessageBox::Ok,
+                             QMessageBox::Ok);
+    }
+
     mEditor->layers()->notifyAnimationLengthChanged();
 
     progress.close();
@@ -754,6 +840,60 @@ void MainWindow2::importImageSequence()
     mIsImportingImageSequence = false;
 }
 
+void MainWindow2::importGIF()
+{
+    auto gifDialog = new ImportImageSeqDialog(this, ImportExportDialog::Import, FileType::GIF);
+    gifDialog->exec();
+    if (gifDialog->result() == QDialog::Rejected)
+    {
+        return;
+    }
+
+    // Flag this so we don't prompt the user about auto-save in the middle of the import.
+    mIsImportingImageSequence = true;
+
+    int space = gifDialog->getSpace();
+
+    // Show a progress dialog, as this could take a while if the gif is huge
+    QProgressDialog progress(tr("Importing Animated GIF..."), tr("Abort"), 0, 100, this);
+    hideQuestionMark(progress);
+    progress.setWindowModality(Qt::WindowModal);
+    progress.show();
+
+    bool failedImport = false;
+    QString strImgFileLower = gifDialog->getFilePath();
+
+    if (strImgFileLower.endsWith(".gif"))
+    {
+        mEditor->importGIF(strImgFileLower, space);
+
+        progress.setValue(50);
+        QApplication::processEvents();  // Required to make progress bar update on-screen.
+
+    } else {
+        if (!failedImport)
+        {
+            failedImport = true;
+        }
+    }
+
+    if (failedImport)
+    {
+        QMessageBox::warning(this,
+                             tr("Warning"),
+                             tr("was unable to import") + strImgFileLower,
+                             QMessageBox::Ok,
+                             QMessageBox::Ok);
+    }
+
+    mEditor->layers()->notifyAnimationLengthChanged();
+
+    progress.setValue(100);
+    progress.close();
+
+    mIsImportingImageSequence = false;
+}
+
 void MainWindow2::importMovie()
 {
     FileDialog fileDialog(this);
@@ -769,7 +909,8 @@ void MainWindow2::lockWidgets(bool shouldLock)
 {
     QDockWidget::DockWidgetFeature feat = shouldLock ? QDockWidget::DockWidgetClosable : QDockWidget::AllDockWidgetFeatures;
 
-    mColorWheel->setFeatures(feat);
+    mColorBox->setFeatures(feat);
+    mColorInspector->setFeatures(feat);
     mColorPalette->setFeatures(feat);
     mDisplayOptionWidget->setFeatures(feat);
     mToolOptions->setFeatures(feat);
@@ -936,10 +1077,11 @@ void MainWindow2::setupKeyboardShortcuts()
 
     mToolBox->toggleViewAction()->setShortcut(cmdKeySeq(CMD_TOGGLE_TOOLBOX));
     mToolOptions->toggleViewAction()->setShortcut(cmdKeySeq(CMD_TOGGLE_TOOL_OPTIONS));
-    mColorWheel->toggleViewAction()->setShortcut(cmdKeySeq(CMD_TOGGLE_COLOR_WHEEL));
+    mColorBox->toggleViewAction()->setShortcut(cmdKeySeq(CMD_TOGGLE_COLOR_WHEEL));
     mColorPalette->toggleViewAction()->setShortcut(cmdKeySeq(CMD_TOGGLE_COLOR_LIBRARY));
     mTimeLine->toggleViewAction()->setShortcut(cmdKeySeq(CMD_TOGGLE_TIMELINE));
     mDisplayOptionWidget->toggleViewAction()->setShortcut(cmdKeySeq(CMD_TOGGLE_DISPLAY_OPTIONS));
+    mColorInspector->toggleViewAction()->setShortcut(cmdKeySeq(CMD_TOGGLE_COLOR_INSPECTOR));
 
     ui->actionHelp->setShortcut(cmdKeySeq(CMD_HELP));
     ui->actionExit->setShortcut(cmdKeySeq(CMD_EXIT));
@@ -954,7 +1096,7 @@ void MainWindow2::clearKeyboardShortcuts()
     }
 }
 
-void MainWindow2::undoActSetText(void)
+void MainWindow2::undoActSetText()
 {
     if (mEditor->mBackupIndex < 0)
     {
@@ -983,7 +1125,7 @@ void MainWindow2::undoActSetText(void)
     }
 }
 
-void MainWindow2::undoActSetEnabled(void)
+void MainWindow2::undoActSetEnabled()
 {
     ui->actionUndo->setEnabled(true);
     ui->actionRedo->setEnabled(true);
@@ -1022,11 +1164,18 @@ void MainWindow2::makeConnections(Editor* editor, ColorBox* colorBox)
     connect(editor->color(), &ColorManager::colorChanged, colorBox, &ColorBox::setColor);
 }
 
+void MainWindow2::makeConnections(Editor* editor, ColorInspector* colorInspector)
+{
+    connect(colorInspector, &ColorInspector::colorChanged, editor->color(), &ColorManager::setColor);
+    connect(editor->color(), &ColorManager::colorChanged, colorInspector, &ColorInspector::setColor);
+}
+
 void MainWindow2::makeConnections(Editor* editor, ScribbleArea* scribbleArea)
 {
     connect(editor->tools(), &ToolManager::toolChanged, scribbleArea, &ScribbleArea::setCurrentTool);
     connect(editor->tools(), &ToolManager::toolPropertyChanged, scribbleArea, &ScribbleArea::updateToolCursor);
     connect(editor->layers(), &LayerManager::currentLayerChanged, scribbleArea, &ScribbleArea::updateAllFrames);
+    connect(editor->layers(), &LayerManager::layerDeleted, scribbleArea, &ScribbleArea::updateAllFrames);
 
     connect(editor, &Editor::currentFrameChanged, scribbleArea, &ScribbleArea::updateFrame);
 
@@ -1051,6 +1200,7 @@ void MainWindow2::makeConnections(Editor* pEditor, TimeLine* pTimeline)
     connect(pTimeline, &TimeLine::newCameraLayer, mCommands, &ActionCommands::addNewCameraLayer);
 
     connect(pTimeline, &TimeLine::toogleAbsoluteOnionClick, pEditor, &Editor::toogleOnionSkinType);
+    connect(mTimeLine, &TimeLine::playButtonTriggered, mCommands, &ActionCommands::PlayStop);
 
     connect(pEditor->layers(), &LayerManager::currentLayerChanged, pTimeline, &TimeLine::updateUI);
     connect(pEditor->layers(), &LayerManager::layerCountChanged, pTimeline, &TimeLine::updateUI);
