@@ -45,7 +45,7 @@ Q_SIGNALS:
     void colorNumberChanged(int);
 
 private:
-    QColor mCurrentFrontColor{ 33, 33, 33 };
+    QColor mCurrentFrontColor{ 33, 33, 33, 255 };
     int  mCurrentColorIndex = 0;
     bool mIsWorkingOnVectorLayer = false;
 };
