@@ -18,6 +18,7 @@ GNU General Public License for more details.
 #include "handtool.h"
 
 #include <cmath>
+#include <QtMath>
 #include <QPixmap>
 #include <QVector2D>
 #include <QMouseEvent>
@@ -39,7 +40,7 @@ void HandTool::loadSettings()
     properties.width = -1;
     properties.feather = -1;
     properties.useFeather = false;
-    properties.inpolLevel = -1;
+    properties.stabilizerLevel = -1;
     properties.useAA = -1;
 }
 
