@@ -20,6 +20,7 @@ GNU General Public License for more details.
 
 #include "basetool.h"
 
+class Layer;
 
 class SelectTool : public BaseTool
 {
@@ -31,11 +32,30 @@ public:
     void loadSettings() override;
     QCursor cursor() override;
 
+private:
+
     void mousePressEvent(QMouseEvent*) override;
     void mouseReleaseEvent(QMouseEvent*) override;
     void mouseMoveEvent(QMouseEvent*) override;
-
     bool keyPressEvent(QKeyEvent *event) override;
+
+    QPointF whichAnchorPoint();
+    void controlOffsetOrigin();
+
+    void beginSelection();
+    void keepSelection();
+
+    inline bool isSelectionPointValid() { return mAnchorOriginPoint != getLastPoint(); }
+    bool maybeDeselect();
+
+
+    // Store selection origin so we can calculate
+    // the selection rectangle in mousePressEvent.
+    QPointF mAnchorOriginPoint;
+    MoveMode mOldMoveMode;
+
+    Layer* mCurrentLayer = nullptr;
+
 };
 
 #endif
